import csle_collector.ryu_manager.ryu_manager_pb2_grpc
import csle_collector.ryu_manager.ryu_manager_pb2
import csle_collector.constants.constants as constants


def get_ryu_status(stub: csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerStub,
                   timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
    """
    Queries the server for the ryu server status

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a RyuDTO describing the status of the ryu controller
    """
    get_ryu_status_msg = csle_collector.ryu_manager.ryu_manager_pb2.GetRyuStatusMsg()
    ryu_dto = stub.getRyuStatus(get_ryu_status_msg, timeout=timeout)
    return ryu_dto


def stop_ryu(stub: csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerStub,
             timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
    """
    Sends a request to the Ryu controller to stop the Ryu controller

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a RyuDTO describing the status of the ryu controller
    """
    stop_ryu_msg = csle_collector.ryu_manager.ryu_manager_pb2.StopRyuMsg()
    ryu_dto = stub.stopRyu(stop_ryu_msg, timeout=timeout)
    return ryu_dto


def start_ryu(stub: csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerStub,
              port: int, web_port: int, controller: str, timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
    """
    Sends a request to the Ryu manager to start the Ryu controller

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :param port: the port that Ryu will listen to
    :param web_port: the port that the Ryu web interface will listen to
    :param controller: the controller that Ryu will start
    :return: a RyuDTO describing the status of the ryu controller
    """
    start_ryu_msg = csle_collector.ryu_manager.ryu_manager_pb2.StartRyuMsg(
        port=port, web_port=web_port, controller=controller)
    ryu_dto = stub.startRyu(start_ryu_msg, timeout=timeout)
    return ryu_dto


def stop_ryu_monitor(stub: csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerStub,
                     timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
    """
    Sends a request to the Ryu controller to stop the Ryu monitor

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a RyuDTO describing the status of the ryu controller
    """
    stop_ryu_monitor_msg = csle_collector.ryu_manager.ryu_manager_pb2.StopRyuMonitorMsg()
    ryu_dto = stub.stopRyuMonitor(stop_ryu_monitor_msg, timeout=timeout)
    return ryu_dto


def start_ryu_monitor(stub: csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerStub,
                      kafka_ip: str, kafka_port: int, time_step_len: int, timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
    """
    Sends a request to the Ryu manager to start the Ryu controller

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :param kafka_port: the port of Kafka to send data to
    :param kafka_ip: the ip of Kafka to send data to
    :param time_step_len: the time period for monitoring
    :return: a RyuDTO describing the status of the ryu controller
    """
    start_ryu_monitor_msg = csle_collector.ryu_manager.ryu_manager_pb2.StartRyuMonitorMsg(
        kafka_ip=kafka_ip, kafka_port=kafka_port, time_step_len=time_step_len)
    ryu_dto = stub.startRyuMonitor(start_ryu_monitor_msg, timeout=timeout)
    return ryu_dto



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import csle_collector.ossec_ids_manager.ossec_ids_manager_pb2 as ossec__ids__manager__pb2


class OSSECIdsManagerStub(object):
    """Interface exported by the server
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.getOSSECIdsAlerts = channel.unary_unary(
                '/OSSECIdsManager/getOSSECIdsAlerts',
                request_serializer=ossec__ids__manager__pb2.GetOSSECIdsAlertsMsg.SerializeToString,
                response_deserializer=ossec__ids__manager__pb2.OSSECIdsLogDTO.FromString,
                )
        self.stopOSSECIdsMonitor = channel.unary_unary(
                '/OSSECIdsManager/stopOSSECIdsMonitor',
                request_serializer=ossec__ids__manager__pb2.StopOSSECIdsMonitorMsg.SerializeToString,
                response_deserializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
                )
        self.startOSSECIdsMonitor = channel.unary_unary(
                '/OSSECIdsManager/startOSSECIdsMonitor',
                request_serializer=ossec__ids__manager__pb2.StartOSSECIdsMonitorMsg.SerializeToString,
                response_deserializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
                )
        self.stopOSSECIds = channel.unary_unary(
                '/OSSECIdsManager/stopOSSECIds',
                request_serializer=ossec__ids__manager__pb2.StopOSSECIdsMsg.SerializeToString,
                response_deserializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
                )
        self.startOSSECIds = channel.unary_unary(
                '/OSSECIdsManager/startOSSECIds',
                request_serializer=ossec__ids__manager__pb2.StartOSSECIdsMsg.SerializeToString,
                response_deserializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
                )
        self.getOSSECIdsMonitorStatus = channel.unary_unary(
                '/OSSECIdsManager/getOSSECIdsMonitorStatus',
                request_serializer=ossec__ids__manager__pb2.GetOSSECIdsMonitorStatusMsg.SerializeToString,
                response_deserializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
                )


class OSSECIdsManagerServicer(object):
    """Interface exported by the server
    """

    def getOSSECIdsAlerts(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopOSSECIdsMonitor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startOSSECIdsMonitor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopOSSECIds(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startOSSECIds(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getOSSECIdsMonitorStatus(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_OSSECIdsManagerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'getOSSECIdsAlerts': grpc.unary_unary_rpc_method_handler(
                    servicer.getOSSECIdsAlerts,
                    request_deserializer=ossec__ids__manager__pb2.GetOSSECIdsAlertsMsg.FromString,
                    response_serializer=ossec__ids__manager__pb2.OSSECIdsLogDTO.SerializeToString,
            ),
            'stopOSSECIdsMonitor': grpc.unary_unary_rpc_method_handler(
                    servicer.stopOSSECIdsMonitor,
                    request_deserializer=ossec__ids__manager__pb2.StopOSSECIdsMonitorMsg.FromString,
                    response_serializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.SerializeToString,
            ),
            'startOSSECIdsMonitor': grpc.unary_unary_rpc_method_handler(
                    servicer.startOSSECIdsMonitor,
                    request_deserializer=ossec__ids__manager__pb2.StartOSSECIdsMonitorMsg.FromString,
                    response_serializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.SerializeToString,
            ),
            'stopOSSECIds': grpc.unary_unary_rpc_method_handler(
                    servicer.stopOSSECIds,
                    request_deserializer=ossec__ids__manager__pb2.StopOSSECIdsMsg.FromString,
                    response_serializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.SerializeToString,
            ),
            'startOSSECIds': grpc.unary_unary_rpc_method_handler(
                    servicer.startOSSECIds,
                    request_deserializer=ossec__ids__manager__pb2.StartOSSECIdsMsg.FromString,
                    response_serializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.SerializeToString,
            ),
            'getOSSECIdsMonitorStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.getOSSECIdsMonitorStatus,
                    request_deserializer=ossec__ids__manager__pb2.GetOSSECIdsMonitorStatusMsg.FromString,
                    response_serializer=ossec__ids__manager__pb2.OSSECIdsMonitorDTO.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'OSSECIdsManager', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class OSSECIdsManager(object):
    """Interface exported by the server
    """

    @staticmethod
    def getOSSECIdsAlerts(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/OSSECIdsManager/getOSSECIdsAlerts',
            ossec__ids__manager__pb2.GetOSSECIdsAlertsMsg.SerializeToString,
            ossec__ids__manager__pb2.OSSECIdsLogDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopOSSECIdsMonitor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/OSSECIdsManager/stopOSSECIdsMonitor',
            ossec__ids__manager__pb2.StopOSSECIdsMonitorMsg.SerializeToString,
            ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startOSSECIdsMonitor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/OSSECIdsManager/startOSSECIdsMonitor',
            ossec__ids__manager__pb2.StartOSSECIdsMonitorMsg.SerializeToString,
            ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopOSSECIds(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/OSSECIdsManager/stopOSSECIds',
            ossec__ids__manager__pb2.StopOSSECIdsMsg.SerializeToString,
            ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startOSSECIds(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/OSSECIdsManager/startOSSECIds',
            ossec__ids__manager__pb2.StartOSSECIdsMsg.SerializeToString,
            ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def getOSSECIdsMonitorStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/OSSECIdsManager/getOSSECIdsMonitorStatus',
            ossec__ids__manager__pb2.GetOSSECIdsMonitorStatusMsg.SerializeToString,
            ossec__ids__manager__pb2.OSSECIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ossec_ids_manager.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17ossec_ids_manager.proto\"@\n\x14GetOSSECIdsAlertsMsg\x12\x11\n\ttimestamp\x18\x01 \x01(\x02\x12\x15\n\rlog_file_path\x18\x02 \x01(\t\"\x18\n\x16StopOSSECIdsMonitorMsg\"\x11\n\x0fStopOSSECIdsMsg\"\x12\n\x10StartOSSECIdsMsg\"u\n\x17StartOSSECIdsMonitorMsg\x12\x10\n\x08kafka_ip\x18\x01 \x01(\t\x12\x12\n\nkafka_port\x18\x02 \x01(\x05\x12\x15\n\rlog_file_path\x18\x03 \x01(\t\x12\x1d\n\x15time_step_len_seconds\x18\x04 \x01(\x05\"\x1d\n\x1bGetOSSECIdsMonitorStatusMsg\"H\n\x12OSSECIdsMonitorDTO\x12\x17\n\x0fmonitor_running\x18\x01 \x01(\x08\x12\x19\n\x11ossec_ids_running\x18\x02 \x01(\x08\"\x80\x07\n\x0eOSSECIdsLogDTO\x12\x11\n\ttimestamp\x18\x01 \x01(\x02\x12\n\n\x02ip\x18\x02 \x01(\t\x12\x1e\n\x16\x61ttempted_admin_alerts\x18\x03 \x01(\x05\x12\x14\n\x0ctotal_alerts\x18\x04 \x01(\x05\x12\x16\n\x0ewarning_alerts\x18\x05 \x01(\x05\x12\x15\n\rsevere_alerts\x18\x06 \x01(\x05\x12 \n\x18\x61lerts_weighted_by_level\x18\x07 \x01(\x05\x12\x16\n\x0elevel_0_alerts\x18\x08 \x01(\x05\x12\x16\n\x0elevel_1_alerts\x18\t \x01(\x05\x12\x16\n\x0elevel_2_alerts\x18\n \x01(\x05\x12\x16\n\x0elevel_3_alerts\x18\x0b \x01(\x05\x12\x16\n\x0elevel_4_alerts\x18\x0c \x01(\x05\x12\x16\n\x0elevel_5_alerts\x18\r \x01(\x05\x12\x16\n\x0elevel_6_alerts\x18\x0e \x01(\x05\x12\x16\n\x0elevel_7_alerts\x18\x0f \x01(\x05\x12\x16\n\x0elevel_8_alerts\x18\x10 \x01(\x05\x12\x16\n\x0elevel_9_alerts\x18\x11 \x01(\x05\x12\x17\n\x0flevel_10_alerts\x18\x12 \x01(\x05\x12\x17\n\x0flevel_11_alerts\x18\x13 \x01(\x05\x12\x17\n\x0flevel_12_alerts\x18\x14 \x01(\x05\x12\x17\n\x0flevel_13_alerts\x18\x15 \x01(\x05\x12\x17\n\x0flevel_14_alerts\x18\x16 \x01(\x05\x12\x17\n\x0flevel_15_alerts\x18\x17 \x01(\x05\x12\x1c\n\x14invalid_login_alerts\x18\x18 \x01(\x05\x12%\n\x1d\x61uthentication_success_alerts\x18\x19 \x01(\x05\x12$\n\x1c\x61uthentication_failed_alerts\x18\x1a \x01(\x05\x12!\n\x19\x63onnection_attempt_alerts\x18\x1b \x01(\x05\x12\x16\n\x0e\x61ttacks_alerts\x18\x1c \x01(\x05\x12\x16\n\x0e\x61\x64\x64user_alerts\x18\x1d \x01(\x05\x12\x13\n\x0bsshd_alerts\x18\x1e \x01(\x05\x12\x12\n\nids_alerts\x18\x1f \x01(\x05\x12\x17\n\x0f\x66irewall_alerts\x18  \x01(\x05\x12\x14\n\x0csquid_alerts\x18! \x01(\x05\x12\x15\n\rapache_alerts\x18\" \x01(\x05\x12\x15\n\rsyslog_alerts\x18# \x01(\x05\x32\xa5\x03\n\x0fOSSECIdsManager\x12=\n\x11getOSSECIdsAlerts\x12\x15.GetOSSECIdsAlertsMsg\x1a\x0f.OSSECIdsLogDTO\"\x00\x12\x45\n\x13stopOSSECIdsMonitor\x12\x17.StopOSSECIdsMonitorMsg\x1a\x13.OSSECIdsMonitorDTO\"\x00\x12G\n\x14startOSSECIdsMonitor\x12\x18.StartOSSECIdsMonitorMsg\x1a\x13.OSSECIdsMonitorDTO\"\x00\x12\x37\n\x0cstopOSSECIds\x12\x10.StopOSSECIdsMsg\x1a\x13.OSSECIdsMonitorDTO\"\x00\x12\x39\n\rstartOSSECIds\x12\x11.StartOSSECIdsMsg\x1a\x13.OSSECIdsMonitorDTO\"\x00\x12O\n\x18getOSSECIdsMonitorStatus\x12\x1c.GetOSSECIdsMonitorStatusMsg\x1a\x13.OSSECIdsMonitorDTO\"\x00\x62\x06proto3')



_GETOSSECIDSALERTSMSG = DESCRIPTOR.message_types_by_name['GetOSSECIdsAlertsMsg']
_STOPOSSECIDSMONITORMSG = DESCRIPTOR.message_types_by_name['StopOSSECIdsMonitorMsg']
_STOPOSSECIDSMSG = DESCRIPTOR.message_types_by_name['StopOSSECIdsMsg']
_STARTOSSECIDSMSG = DESCRIPTOR.message_types_by_name['StartOSSECIdsMsg']
_STARTOSSECIDSMONITORMSG = DESCRIPTOR.message_types_by_name['StartOSSECIdsMonitorMsg']
_GETOSSECIDSMONITORSTATUSMSG = DESCRIPTOR.message_types_by_name['GetOSSECIdsMonitorStatusMsg']
_OSSECIDSMONITORDTO = DESCRIPTOR.message_types_by_name['OSSECIdsMonitorDTO']
_OSSECIDSLOGDTO = DESCRIPTOR.message_types_by_name['OSSECIdsLogDTO']
GetOSSECIdsAlertsMsg = _reflection.GeneratedProtocolMessageType('GetOSSECIdsAlertsMsg', (_message.Message,), {
  'DESCRIPTOR' : _GETOSSECIDSALERTSMSG,
  '__module__' : 'ossec_ids_manager_pb2'
  # @@protoc_insertion_point(class_scope:GetOSSECIdsAlertsMsg)
  })
_sym_db.RegisterMessage(GetOSSECIdsAlertsMsg)

StopOSSECIdsMonitorMsg = _reflection.GeneratedProtocolMessageType('StopOSSECIdsMonitorMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPOSSECIDSMONITORMSG,
  '__module__' : 'ossec_ids_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopOSSECIdsMonitorMsg)
  })
_sym_db.RegisterMessage(StopOSSECIdsMonitorMsg)

StopOSSECIdsMsg = _reflection.GeneratedProtocolMessageType('StopOSSECIdsMsg', (_message.Message,), {
  'DESCRIPTOR' : _STOPOSSECIDSMSG,
  '__module__' : 'ossec_ids_manager_pb2'
  # @@protoc_insertion_point(class_scope:StopOSSECIdsMsg)
  })
_sym_db.RegisterMessage(StopOSSECIdsMsg)

StartOSSECIdsMsg = _reflection.GeneratedProtocolMessageType('StartOSSECIdsMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTOSSECIDSMSG,
  '__module__' : 'ossec_ids_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartOSSECIdsMsg)
  })
_sym_db.RegisterMessage(StartOSSECIdsMsg)

StartOSSECIdsMonitorMsg = _reflection.GeneratedProtocolMessageType('StartOSSECIdsMonitorMsg', (_message.Message,), {
  'DESCRIPTOR' : _STARTOSSECIDSMONITORMSG,
  '__module__' : 'ossec_ids_manager_pb2'
  # @@protoc_insertion_point(class_scope:StartOSSECIdsMonitorMsg)
  })
_sym_db.RegisterMessage(StartOSSECIdsMonitorMsg)

GetOSSECIdsMonitorStatusMsg = _reflection.GeneratedProtocolMessageType('GetOSSECIdsMonitorStatusMsg', (_message.Message,), {
  'DESCRIPTOR' : _GETOSSECIDSMONITORSTATUSMSG,
  '__module__' : 'ossec_ids_manager_pb2'
  # @@protoc_insertion_point(class_scope:GetOSSECIdsMonitorStatusMsg)
  })
_sym_db.RegisterMessage(GetOSSECIdsMonitorStatusMsg)

OSSECIdsMonitorDTO = _reflection.GeneratedProtocolMessageType('OSSECIdsMonitorDTO', (_message.Message,), {
  'DESCRIPTOR' : _OSSECIDSMONITORDTO,
  '__module__' : 'ossec_ids_manager_pb2'
  # @@protoc_insertion_point(class_scope:OSSECIdsMonitorDTO)
  })
_sym_db.RegisterMessage(OSSECIdsMonitorDTO)

OSSECIdsLogDTO = _reflection.GeneratedProtocolMessageType('OSSECIdsLogDTO', (_message.Message,), {
  'DESCRIPTOR' : _OSSECIDSLOGDTO,
  '__module__' : 'ossec_ids_manager_pb2'
  # @@protoc_insertion_point(class_scope:OSSECIdsLogDTO)
  })
_sym_db.RegisterMessage(OSSECIdsLogDTO)

_OSSECIDSMANAGER = DESCRIPTOR.services_by_name['OSSECIdsManager']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _GETOSSECIDSALERTSMSG._serialized_start=27
  _GETOSSECIDSALERTSMSG._serialized_end=91
  _STOPOSSECIDSMONITORMSG._serialized_start=93
  _STOPOSSECIDSMONITORMSG._serialized_end=117
  _STOPOSSECIDSMSG._serialized_start=119
  _STOPOSSECIDSMSG._serialized_end=136
  _STARTOSSECIDSMSG._serialized_start=138
  _STARTOSSECIDSMSG._serialized_end=156
  _STARTOSSECIDSMONITORMSG._serialized_start=158
  _STARTOSSECIDSMONITORMSG._serialized_end=275
  _GETOSSECIDSMONITORSTATUSMSG._serialized_start=277
  _GETOSSECIDSMONITORSTATUSMSG._serialized_end=306
  _OSSECIDSMONITORDTO._serialized_start=308
  _OSSECIDSMONITORDTO._serialized_end=380
  _OSSECIDSLOGDTO._serialized_start=383
  _OSSECIDSLOGDTO._serialized_end=1279
  _OSSECIDSMANAGER._serialized_start=1282
  _OSSECIDSMANAGER._serialized_end=1703
# @@protoc_insertion_point(module_scope)



import socket
import netifaces
import threading
import logging
import time
from confluent_kafka import Producer
import csle_collector.constants.constants as constants
from csle_collector.client_manager.threads.arrival_thread import ArrivalThread


class ProducerThread(threading.Thread):
    """
    Thread that pushes statistics to Kafka
    """

    def __init__(self, arrival_thread: ArrivalThread, time_step_len_seconds: int, ip: str, port: int):
        """
        Initializes the thread

        :param arrival_thread: the thread that manages the client arrivals, used to extract statistics
        :param time_step_len_seconds: the length between pushing statistics to Kafka
        :param ip: the ip of the Kafka server
        :param port: the port of the Kafka server
        """
        threading.Thread.__init__(self)
        self.arrival_thread = arrival_thread
        self.time_step_len_seconds = time_step_len_seconds
        self.stopped = False
        self.kafka_ip = ip
        self.port = port
        self.hostname = socket.gethostname()
        try:
            self.ip = netifaces.ifaddresses(constants.INTERFACES.ETH0)[netifaces.AF_INET][0][constants.INTERFACES.ADDR]
        except Exception:
            self.ip = socket.gethostbyname(self.hostname)
        self.conf = {
            constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.kafka_ip}:{self.port}",
            constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        self.producer = Producer(**self.conf)
        logging.info(f"Starting producer thread, ip:{self.ip}, kafka port:{self.port}, "
                     f"time_step_len:{self.time_step_len_seconds}, kafka_ip:{self.kafka_ip}")

    def run(self) -> None:
        """
        Main loop of the thread, pushes data to Kafka periodically

        :return: None
        """
        while not self.stopped and self.arrival_thread is not None:
            time.sleep(self.time_step_len_seconds)
            if self.arrival_thread is not None:
                ts = time.time()
                num_clients = len(self.arrival_thread.client_threads)
                rate = self.arrival_thread.rate
                mu = 4
                self.producer.produce(constants.KAFKA_CONFIG.CLIENT_POPULATION_TOPIC_NAME,
                                      f"{ts},{self.ip},{num_clients},{rate},{mu}")
                self.producer.poll(0)


import logging
import socket
import time
import grpc
import threading
import subprocess
import netifaces
from concurrent import futures
from confluent_kafka import Producer
import csle_collector.snort_ids_manager.snort_ids_manager_pb2_grpc
import csle_collector.snort_ids_manager.snort_ids_manager_pb2
from csle_collector.snort_ids_manager.snort_ids_manager_util import SnortIdsManagerUtil
import csle_collector.constants.constants as constants


class SnortIDSMonitorThread(threading.Thread):
    """
    Thread that collects the Snort IDS statistics and pushes it to Kafka periodically
    """

    def __init__(self, kafka_ip: str, kafka_port: int, ip: str, hostname: str, log_file_path: str,
                 time_step_len_seconds: int):
        """
        Initializes the thread

        :param kafka_ip: IP of the Kafka server to push to
        :param kafka_port: port of the Kafka server to push to
        :param ip: ip of the server we are pushing from
        :param hostname: hostname of the server we are pushing from
        :param log_file_path: path to the IDS log
        :param time_step_len_seconds: the length of a timestep
        """
        threading.Thread.__init__(self)
        self.kafka_ip = kafka_ip
        self.kafka_port = kafka_port
        self.ip = ip
        self.hostname = hostname
        self.log_file_path = log_file_path
        self.latest_ts = time.time()
        self.time_step_len_seconds = time_step_len_seconds
        self.conf = {constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.kafka_ip}:{self.kafka_port}",
                     constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        self.producer = Producer(**self.conf)
        self.running = True
        logging.info("SnortIDSMonitor thread started successfully")

    def run(self) -> None:
        """
        Main loop of the thread. Parses the IDS log and pushes it to Kafka periodically

        :return: None
        """
        while self.running:
            time.sleep(self.time_step_len_seconds)
            try:
                agg_alert_counters, rule_alert_counters, ip_alert_counters = \
                    SnortIdsManagerUtil.read_snort_ids_data(self.latest_ts)
                record = agg_alert_counters.to_kafka_record(ip=self.ip)
                self.producer.produce(constants.KAFKA_CONFIG.SNORT_IDS_LOG_TOPIC_NAME, record)
                self.producer.poll(0)
                record = rule_alert_counters.to_kafka_record(ip=self.ip)
                self.producer.produce(constants.KAFKA_CONFIG.SNORT_IDS_RULE_LOG_TOPIC_NAME, record)
                self.producer.poll(0)
                for ip_alert_counter in ip_alert_counters:
                    record = ip_alert_counter.to_kafka_record(ip=self.ip)
                    self.producer.produce(constants.KAFKA_CONFIG.SNORT_IDS_IP_LOG_TOPIC_NAME, record)
                    self.producer.poll(0)
            except Exception as e:
                logging.info(f"There was an exception parsing the Snort logs: {str(e)}, {repr(e)}")
            self.latest_ts = time.time()


class SnortIdsManagerServicer(csle_collector.snort_ids_manager.snort_ids_manager_pb2_grpc.SnortIdsManagerServicer):
    """
    gRPC server for collecting Snort IDS statistics.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        logging.basicConfig(filename=f"{constants.LOG_FILES.SNORT_IDS_MANAGER_LOG_DIR}"
                                     f"{constants.LOG_FILES.SNORT_IDS_MANAGER_LOG_FILE}", level=logging.INFO)
        self.hostname = socket.gethostname()
        try:
            self.ip = netifaces.ifaddresses(constants.INTERFACES.ETH0)[netifaces.AF_INET][0][constants.INTERFACES.ADDR]
        except Exception:
            self.ip = socket.gethostbyname(self.hostname)
        self.conf = {
            constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.ip}:{constants.KAFKA.PORT}",
            constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        self.ids_monitor_thread = None
        logging.info(f"Starting the SnortIDSManager hostname: {self.hostname} ip: {self.ip}")

    def _is_snort_running(self) -> bool:
        """
        Utility method to check if Snort is running

        :return: status and list of topics
        """
        logging.info("Checking if Snort is running")
        ps_res = subprocess.Popen(constants.SNORT_IDS_ROUTER.PS_AUX_CMD.split(" "), stdout=subprocess.PIPE)
        status_output = subprocess.run(constants.SNORT_IDS_ROUTER.GREP_SNORT_CONF.split(" "), stdin=ps_res.stdout,
                                       capture_output=True, text=True).stdout
        ps_res.wait()
        logging.info(f"Output: {status_output}")
        running = constants.SNORT_IDS_ROUTER.SEARCH_SNORT_RUNNING in status_output
        logging.info(f"Running status: {running}")
        return running

    def getSnortIdsAlerts(
            self, request: csle_collector.snort_ids_manager.snort_ids_manager_pb2.GetSnortIdsAlertsMsg,
            context: grpc.ServicerContext) -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsLogDTO:
        """
        Gets the statistics of the IDS log from a given timestamp

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with IDS statistics
        """
        agg_alert_counters, rule_alert_counters, ip_alert_counters = \
            SnortIdsManagerUtil.read_snort_ids_data(request.timestamp)
        ids_log_dto = agg_alert_counters.to_dto(ip=self.ip)
        return ids_log_dto

    def startSnortIdsMonitor(
            self, request: csle_collector.snort_ids_manager.snort_ids_manager_pb2.StartSnortIdsMonitorMsg,
            context: grpc.ServicerContext) -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO:
        """
        Starts the Snort IDS monitor thread

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS monitor thread
        """
        logging.info(f"Starting the SnortIDSMonitor thread, timestep length: {request.time_step_len_seconds}, "
                     f"log file path: {request.log_file_path}, kafka ip: {request.kafka_ip}, "
                     f"kafka port: {request.kafka_port}")
        if self.ids_monitor_thread is not None:
            self.ids_monitor_thread.running = False
        self.ids_monitor_thread = SnortIDSMonitorThread(kafka_ip=request.kafka_ip, kafka_port=request.kafka_port,
                                                        ip=self.ip, hostname=self.hostname,
                                                        log_file_path=request.log_file_path,
                                                        time_step_len_seconds=request.time_step_len_seconds)
        snort_running = self._is_snort_running()
        self.ids_monitor_thread.start()
        logging.info("Started the SnortIDSMonitor thread")
        return csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO(
            monitor_running=True, snort_ids_running=snort_running)

    def startSnortIds(self, request: csle_collector.snort_ids_manager.snort_ids_manager_pb2.StartSnortIdsMsg,
                      context: grpc.ServicerContext) \
            -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO:
        """
        Starts the Snort IDS

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS and its monitor thread
        """
        logging.info(f"Starting the SnortIDS, ingress interface: {request.ingress_interface}, "
                     f"egress interface: {request.egress_interface}, subnetmask: {request.subnetmask}")
        monitor_running = False
        if self.ids_monitor_thread is not None:
            monitor_running = self.ids_monitor_thread.running
        snort_running = self._is_snort_running()
        if snort_running:
            cmd = constants.SNORT_IDS_ROUTER.STOP_SNORT_IDS
            result = subprocess.run(cmd.split(" "),
                                    capture_output=True, text=True)
            logging.info(f"Stopped the Snort IDS, stdout:{result.stdout}, stderr: {result.stderr}, cmd: {cmd}")
        if not snort_running:
            cmd = constants.SNORT_IDS_ROUTER.START_SNORT_IDS.format(request.ingress_interface, request.egress_interface,
                                                                    request.subnetmask)
            result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
            logging.info(f"Started the Snort IDS, stdout:{result.stdout}, stderr: {result.stderr}, cmd: {cmd}")
        logging.info("Started the SnortIDS")
        cmd = constants.SNORT_IDS_ROUTER.SNORT_LOG_DIR_PERMISSION_CMD
        logging.info("Changing permissions of the snort log directory")
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        logging.info(f"Changed the log dir permissions, stdout:{result.stdout}, stderr: {result.stderr}, cmd: {cmd}")
        return csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO(
            monitor_running=monitor_running, snort_ids_running=True)

    def stopSnortIds(self, request: csle_collector.snort_ids_manager.snort_ids_manager_pb2.StartSnortIdsMsg,
                     context: grpc.ServicerContext) \
            -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO:
        """
        Stops the Snort IDS

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS and its monitor thread
        """
        logging.info("Stopping the SnortIDS")
        monitor_running = False
        if self.ids_monitor_thread is not None:
            monitor_running = self.ids_monitor_thread.running
        result = subprocess.run(constants.SNORT_IDS_ROUTER.STOP_SNORT_IDS.split(" "),
                                capture_output=True, text=True)
        logging.info(f"Stopped the SnortIDS, stdout: {result.stdout}, stderr: {result.stderr}")
        return csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO(
            monitor_running=monitor_running, snort_ids_running=False)

    def stopSnortIdsMonitor(self,
                            request: csle_collector.snort_ids_manager.snort_ids_manager_pb2.StopSnortIdsMonitorMsg,
                            context: grpc.ServicerContext) \
            -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO:
        """
        Stops the Snort IDS monitor thread if it is running

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS monitor thread
        """
        if self.ids_monitor_thread is not None:
            self.ids_monitor_thread.running = False
        snort_running = self._is_snort_running()
        return csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO(
            monitor_running=False, snort_ids_running=snort_running)

    def getSnortIdsMonitorStatus(
            self, request: csle_collector.snort_ids_manager.snort_ids_manager_pb2.GetSnortIdsMonitorStatusMsg,
            context: grpc.ServicerContext) -> csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO:
        """
        Gets the status of the Snort IDS Monitor thread

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a DTO with the status of the IDS monitor
        """
        running = False
        if self.ids_monitor_thread is not None:
            running = self.ids_monitor_thread.running
        snort_running = self._is_snort_running()
        return csle_collector.snort_ids_manager.snort_ids_manager_pb2.SnortIdsMonitorDTO(
            monitor_running=running, snort_ids_running=snort_running)


def serve(port: int = 50048, log_dir: str = "/", max_workers: int = 10,
          log_file_name: str = "snort_ids_manager.log") -> None:
    """
    Starts the gRPC server for managing clients

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of GRPC workers
    :return: None
    """
    constants.LOG_FILES.SNORT_IDS_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.SNORT_IDS_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.snort_ids_manager.snort_ids_manager_pb2_grpc.add_SnortIdsManagerServicer_to_server(
        SnortIdsManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"SnortIdsManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import csle_collector.host_manager.host_manager_pb2 as host__manager__pb2


class HostManagerStub(object):
    """Interface exported by the server
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.stopHostMonitor = channel.unary_unary(
                '/HostManager/stopHostMonitor',
                request_serializer=host__manager__pb2.StopHostMonitorMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.startHostMonitor = channel.unary_unary(
                '/HostManager/startHostMonitor',
                request_serializer=host__manager__pb2.StartHostMonitorMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.getHostStatus = channel.unary_unary(
                '/HostManager/getHostStatus',
                request_serializer=host__manager__pb2.GetHostStatusMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.getHostMetrics = channel.unary_unary(
                '/HostManager/getHostMetrics',
                request_serializer=host__manager__pb2.GetHostMetricsMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostMetricsDTO.FromString,
                )
        self.stopFilebeat = channel.unary_unary(
                '/HostManager/stopFilebeat',
                request_serializer=host__manager__pb2.StopFilebeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.startFilebeat = channel.unary_unary(
                '/HostManager/startFilebeat',
                request_serializer=host__manager__pb2.StartFilebeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.configFilebeat = channel.unary_unary(
                '/HostManager/configFilebeat',
                request_serializer=host__manager__pb2.ConfigFilebeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.stopPacketbeat = channel.unary_unary(
                '/HostManager/stopPacketbeat',
                request_serializer=host__manager__pb2.StopPacketbeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.startPacketbeat = channel.unary_unary(
                '/HostManager/startPacketbeat',
                request_serializer=host__manager__pb2.StartPacketbeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.configPacketbeat = channel.unary_unary(
                '/HostManager/configPacketbeat',
                request_serializer=host__manager__pb2.ConfigPacketbeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.stopMetricbeat = channel.unary_unary(
                '/HostManager/stopMetricbeat',
                request_serializer=host__manager__pb2.StopMetricbeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.startMetricbeat = channel.unary_unary(
                '/HostManager/startMetricbeat',
                request_serializer=host__manager__pb2.StartMetricbeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.configMetricbeat = channel.unary_unary(
                '/HostManager/configMetricbeat',
                request_serializer=host__manager__pb2.ConfigMetricbeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.stopHeartbeat = channel.unary_unary(
                '/HostManager/stopHeartbeat',
                request_serializer=host__manager__pb2.StopHeartbeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.startHeartbeat = channel.unary_unary(
                '/HostManager/startHeartbeat',
                request_serializer=host__manager__pb2.StartHeartbeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.configHeartbeat = channel.unary_unary(
                '/HostManager/configHeartbeat',
                request_serializer=host__manager__pb2.ConfigHeartbeatMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.stopSpark = channel.unary_unary(
                '/HostManager/stopSpark',
                request_serializer=host__manager__pb2.StopSparkMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )
        self.startSpark = channel.unary_unary(
                '/HostManager/startSpark',
                request_serializer=host__manager__pb2.StartSparkMsg.SerializeToString,
                response_deserializer=host__manager__pb2.HostStatusDTO.FromString,
                )


class HostManagerServicer(object):
    """Interface exported by the server
    """

    def stopHostMonitor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startHostMonitor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getHostStatus(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getHostMetrics(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopFilebeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startFilebeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def configFilebeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopPacketbeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startPacketbeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def configPacketbeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopMetricbeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startMetricbeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def configMetricbeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopHeartbeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startHeartbeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def configHeartbeat(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopSpark(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startSpark(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_HostManagerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'stopHostMonitor': grpc.unary_unary_rpc_method_handler(
                    servicer.stopHostMonitor,
                    request_deserializer=host__manager__pb2.StopHostMonitorMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'startHostMonitor': grpc.unary_unary_rpc_method_handler(
                    servicer.startHostMonitor,
                    request_deserializer=host__manager__pb2.StartHostMonitorMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'getHostStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.getHostStatus,
                    request_deserializer=host__manager__pb2.GetHostStatusMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'getHostMetrics': grpc.unary_unary_rpc_method_handler(
                    servicer.getHostMetrics,
                    request_deserializer=host__manager__pb2.GetHostMetricsMsg.FromString,
                    response_serializer=host__manager__pb2.HostMetricsDTO.SerializeToString,
            ),
            'stopFilebeat': grpc.unary_unary_rpc_method_handler(
                    servicer.stopFilebeat,
                    request_deserializer=host__manager__pb2.StopFilebeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'startFilebeat': grpc.unary_unary_rpc_method_handler(
                    servicer.startFilebeat,
                    request_deserializer=host__manager__pb2.StartFilebeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'configFilebeat': grpc.unary_unary_rpc_method_handler(
                    servicer.configFilebeat,
                    request_deserializer=host__manager__pb2.ConfigFilebeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'stopPacketbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.stopPacketbeat,
                    request_deserializer=host__manager__pb2.StopPacketbeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'startPacketbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.startPacketbeat,
                    request_deserializer=host__manager__pb2.StartPacketbeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'configPacketbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.configPacketbeat,
                    request_deserializer=host__manager__pb2.ConfigPacketbeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'stopMetricbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.stopMetricbeat,
                    request_deserializer=host__manager__pb2.StopMetricbeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'startMetricbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.startMetricbeat,
                    request_deserializer=host__manager__pb2.StartMetricbeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'configMetricbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.configMetricbeat,
                    request_deserializer=host__manager__pb2.ConfigMetricbeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'stopHeartbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.stopHeartbeat,
                    request_deserializer=host__manager__pb2.StopHeartbeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'startHeartbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.startHeartbeat,
                    request_deserializer=host__manager__pb2.StartHeartbeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'configHeartbeat': grpc.unary_unary_rpc_method_handler(
                    servicer.configHeartbeat,
                    request_deserializer=host__manager__pb2.ConfigHeartbeatMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'stopSpark': grpc.unary_unary_rpc_method_handler(
                    servicer.stopSpark,
                    request_deserializer=host__manager__pb2.StopSparkMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
            'startSpark': grpc.unary_unary_rpc_method_handler(
                    servicer.startSpark,
                    request_deserializer=host__manager__pb2.StartSparkMsg.FromString,
                    response_serializer=host__manager__pb2.HostStatusDTO.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'HostManager', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class HostManager(object):
    """Interface exported by the server
    """

    @staticmethod
    def stopHostMonitor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/stopHostMonitor',
            host__manager__pb2.StopHostMonitorMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startHostMonitor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/startHostMonitor',
            host__manager__pb2.StartHostMonitorMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def getHostStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/getHostStatus',
            host__manager__pb2.GetHostStatusMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def getHostMetrics(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/getHostMetrics',
            host__manager__pb2.GetHostMetricsMsg.SerializeToString,
            host__manager__pb2.HostMetricsDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopFilebeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/stopFilebeat',
            host__manager__pb2.StopFilebeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startFilebeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/startFilebeat',
            host__manager__pb2.StartFilebeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def configFilebeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/configFilebeat',
            host__manager__pb2.ConfigFilebeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopPacketbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/stopPacketbeat',
            host__manager__pb2.StopPacketbeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startPacketbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/startPacketbeat',
            host__manager__pb2.StartPacketbeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def configPacketbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/configPacketbeat',
            host__manager__pb2.ConfigPacketbeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopMetricbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/stopMetricbeat',
            host__manager__pb2.StopMetricbeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startMetricbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/startMetricbeat',
            host__manager__pb2.StartMetricbeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def configMetricbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/configMetricbeat',
            host__manager__pb2.ConfigMetricbeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopHeartbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/stopHeartbeat',
            host__manager__pb2.StopHeartbeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startHeartbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/startHeartbeat',
            host__manager__pb2.StartHeartbeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def configHeartbeat(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/configHeartbeat',
            host__manager__pb2.ConfigHeartbeatMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopSpark(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/stopSpark',
            host__manager__pb2.StopSparkMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startSpark(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/HostManager/startSpark',
            host__manager__pb2.StartSparkMsg.SerializeToString,
            host__manager__pb2.HostStatusDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: client_manager.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x63lient_manager.proto\"\x10\n\x0eStopClientsMsg\"|\n\x0fStartClientsMsg\x12\x1d\n\x15time_step_len_seconds\x18\x01 \x01(\x05\x12-\n\x10workflows_config\x18\x02 \x01(\x0b\x32\x13.WorkflowsConfigDTO\x12\x1b\n\x07\x63lients\x18\x03 \x03(\x0b\x32\n.ClientDTO\"\x94\x03\n\tClientDTO\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x1d\n\x15workflow_distribution\x18\x02 \x03(\x02\x12:\n\x17\x63onstant_arrival_config\x18\x03 \x01(\x0b\x32\x19.ConstantArrivalConfigDTO\x12\x32\n\x13sine_arrival_config\x18\x04 \x01(\x0b\x32\x15.SineArrivalConfigDTO\x12\x38\n\x16spiking_arrival_config\x18\x05 \x01(\x0b\x32\x18.SpikingArrivalConfigDTO\x12N\n\"piece_wise_constant_arrival_config\x18\x06 \x01(\x0b\x32\".PieceWiseConstantArrivalConfigDTO\x12\x34\n\x14\x65ptmp_arrival_config\x18\x07 \x01(\x0b\x32\x16.EPTMPArrivalConfigDTO\x12\n\n\x02mu\x18\t \x01(\x02\x12 \n\x18\x65xponential_service_time\x18\n \x01(\x08\"\x0f\n\rGetClientsMsg\"K\n\x10StartProducerMsg\x12\n\n\x02ip\x18\x01 \x01(\t\x12\x0c\n\x04port\x18\x02 \x01(\x05\x12\x1d\n\x15time_step_len_seconds\x18\x03 \x01(\x05\"\x11\n\x0fStopProducerMsg\"3\n\x1aProbabilityDistributionDTO\x12\x15\n\rprobabilities\x18\x01 \x03(\x02\"@\n\x13TransitionMatrixDTO\x12)\n\x04rows\x18\x01 \x03(\x0b\x32\x1b.ProbabilityDistributionDTO\"l\n\x16WorkflowMarkovChainDTO\x12/\n\x11transition_matrix\x18\x01 \x01(\x0b\x32\x14.TransitionMatrixDTO\x12\x15\n\rinitial_state\x18\x02 \x01(\x05\x12\n\n\x02id\x18\x03 \x01(\x05\"Q\n\x12WorkflowServiceDTO\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0b\n\x03ips\x18\x02 \x03(\t\x12\"\n\x08\x63ommands\x18\x03 \x03(\x0b\x32\x10.NodeCommandsDTO\"#\n\x0fNodeCommandsDTO\x12\x10\n\x08\x63ommands\x18\x01 \x03(\t\"}\n\x12WorkflowsConfigDTO\x12\x37\n\x16workflow_markov_chains\x18\x01 \x03(\x0b\x32\x17.WorkflowMarkovChainDTO\x12.\n\x11workflow_services\x18\x02 \x03(\x0b\x32\x13.WorkflowServiceDTO\"\xa8\x01\n\nClientsDTO\x12\x13\n\x0bnum_clients\x18\x01 \x01(\x05\x12\x1d\n\x15\x63lient_process_active\x18\x02 \x01(\x08\x12\x17\n\x0fproducer_active\x18\x03 \x01(\x08\x12%\n\x1d\x63lients_time_step_len_seconds\x18\x04 \x01(\x05\x12&\n\x1eproducer_time_step_len_seconds\x18\x05 \x01(\x05\"(\n\x18\x43onstantArrivalConfigDTO\x12\x0c\n\x04lamb\x18\x01 \x01(\x02\"`\n\x14SineArrivalConfigDTO\x12\x0c\n\x04lamb\x18\x01 \x01(\x02\x12\x1b\n\x13time_scaling_factor\x18\x02 \x01(\x02\x12\x1d\n\x15period_scaling_factor\x18\x03 \x01(\x02\"=\n\x17SpikingArrivalConfigDTO\x12\x11\n\texponents\x18\x01 \x03(\x02\x12\x0f\n\x07\x66\x61\x63tors\x18\x02 \x03(\x02\"M\n!PieceWiseConstantArrivalConfigDTO\x12\x13\n\x0b\x62reakvalues\x18\x01 \x03(\x02\x12\x13\n\x0b\x62reakpoints\x18\x02 \x03(\x05\"U\n\x15\x45PTMPArrivalConfigDTO\x12\x0e\n\x06thetas\x18\x01 \x03(\x02\x12\x0e\n\x06gammas\x18\x02 \x03(\x02\x12\x0c\n\x04phis\x18\x03 \x03(\x02\x12\x0e\n\x06omegas\x18\x04 \x03(\x02\x32\x80\x02\n\rClientManager\x12+\n\ngetClients\x12\x0e.GetClientsMsg\x1a\x0b.ClientsDTO\"\x00\x12-\n\x0bstopClients\x12\x0f.StopClientsMsg\x1a\x0b.ClientsDTO\"\x00\x12/\n\x0cstartClients\x12\x10.StartClientsMsg\x1a\x0b.ClientsDTO\"\x00\x12\x31\n\rstartProducer\x12\x11.StartProducerMsg\x1a\x0b.ClientsDTO\"\x00\x12/\n\x0cstopProducer\x12\x10.StopProducerMsg\x1a\x0b.ClientsDTO\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'client_manager_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STOPCLIENTSMSG._serialized_start=24
  _STOPCLIENTSMSG._serialized_end=40
  _STARTCLIENTSMSG._serialized_start=42
  _STARTCLIENTSMSG._serialized_end=166
  _CLIENTDTO._serialized_start=169
  _CLIENTDTO._serialized_end=573
  _GETCLIENTSMSG._serialized_start=575
  _GETCLIENTSMSG._serialized_end=590
  _STARTPRODUCERMSG._serialized_start=592
  _STARTPRODUCERMSG._serialized_end=667
  _STOPPRODUCERMSG._serialized_start=669
  _STOPPRODUCERMSG._serialized_end=686
  _PROBABILITYDISTRIBUTIONDTO._serialized_start=688
  _PROBABILITYDISTRIBUTIONDTO._serialized_end=739
  _TRANSITIONMATRIXDTO._serialized_start=741
  _TRANSITIONMATRIXDTO._serialized_end=805
  _WORKFLOWMARKOVCHAINDTO._serialized_start=807
  _WORKFLOWMARKOVCHAINDTO._serialized_end=915
  _WORKFLOWSERVICEDTO._serialized_start=917
  _WORKFLOWSERVICEDTO._serialized_end=998
  _NODECOMMANDSDTO._serialized_start=1000
  _NODECOMMANDSDTO._serialized_end=1035
  _WORKFLOWSCONFIGDTO._serialized_start=1037
  _WORKFLOWSCONFIGDTO._serialized_end=1162
  _CLIENTSDTO._serialized_start=1165
  _CLIENTSDTO._serialized_end=1333
  _CONSTANTARRIVALCONFIGDTO._serialized_start=1335
  _CONSTANTARRIVALCONFIGDTO._serialized_end=1375
  _SINEARRIVALCONFIGDTO._serialized_start=1377
  _SINEARRIVALCONFIGDTO._serialized_end=1473
  _SPIKINGARRIVALCONFIGDTO._serialized_start=1475
  _SPIKINGARRIVALCONFIGDTO._serialized_end=1536
  _PIECEWISECONSTANTARRIVALCONFIGDTO._serialized_start=1538
  _PIECEWISECONSTANTARRIVALCONFIGDTO._serialized_end=1615
  _EPTMPARRIVALCONFIGDTO._serialized_start=1617
  _EPTMPARRIVALCONFIGDTO._serialized_end=1702
  _CLIENTMANAGER._serialized_start=1705
  _CLIENTMANAGER._serialized_end=1961
# @@protoc_insertion_point(module_scope)



import csle_collector.kafka_manager.kafka_manager_pb2_grpc
import csle_collector.kafka_manager.kafka_manager_pb2
import csle_collector.constants.constants as constants


def get_kafka_status(stub: csle_collector.kafka_manager.kafka_manager_pb2_grpc.KafkaManagerStub,
                     timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO:
    """
    Queries the server for the kafka server status

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a KafkaDTO describing the status of the kafka server
    """
    get_kafka_status_msg = csle_collector.kafka_manager.kafka_manager_pb2.GetKafkaStatusMsg()
    kafka_dto = stub.getKafkaStatus(get_kafka_status_msg, timeout=timeout)
    return kafka_dto


def create_topic(stub: csle_collector.kafka_manager.kafka_manager_pb2_grpc.KafkaManagerStub,
                 name: str, partitions: int, replicas: int, retention_time_hours: int,
                 timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO:
    """
    Sends a request to the KafkaManager to create a new Kafka topic

    :param stub: the stub to send the remote gRPC to the server
    :param name: the name of the Kafka topic
    :param partitions: the number of partitions of the Kafka topic
    :param replicas: the number of replicas of the Kafka topic
    :param retention_time_hours: the retention time for the topic
    :param timeout: the GRPC timeout (seconds)
    :return: a KafkaDTO describing the status of the kafka server
    """
    create_kafka_topic_msg = csle_collector.kafka_manager.kafka_manager_pb2.CreateTopicMsg(
        name=name, partitions=partitions, replicas=replicas, retention_time_hours=retention_time_hours)
    kafka_dto = stub.createTopic(create_kafka_topic_msg, timeout=timeout)
    return kafka_dto


def stop_kafka(stub: csle_collector.kafka_manager.kafka_manager_pb2_grpc.KafkaManagerStub,
               timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO:
    """
    Sends a request to the Kafka server to stop the Kafka server

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a KafkaDTO describing the status of the kafka server
    """
    stop_kafka_msg = csle_collector.kafka_manager.kafka_manager_pb2.StopKafkaMsg()
    kafka_dto = stub.stopKafka(stop_kafka_msg, timeout=timeout)
    return kafka_dto


def start_kafka(stub: csle_collector.kafka_manager.kafka_manager_pb2_grpc.KafkaManagerStub,
                timeout=constants.GRPC.TIMEOUT_SECONDS) \
        -> csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO:
    """
    Sends a request to the Kafka server to start the Kafka server

    :param stub: the stub to send the remote gRPC to the server
    :param timeout: the GRPC timeout (seconds)
    :return: a KafkaDTO describing the status of the kafka server
    """
    start_kafka_msg = csle_collector.kafka_manager.kafka_manager_pb2.StartKafkaMsg()
    kafka_dto = stub.startKafka(start_kafka_msg, timeout=timeout)
    return kafka_dto



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import csle_collector.docker_stats_manager.docker_stats_manager_pb2 as docker__stats__manager__pb2


class DockerStatsManagerStub(object):
    """Interface exported by the server
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.getDockerStatsMonitorStatus = channel.unary_unary(
                '/DockerStatsManager/getDockerStatsMonitorStatus',
                request_serializer=docker__stats__manager__pb2.GetDockerStatsMonitorStatusMsg.SerializeToString,
                response_deserializer=docker__stats__manager__pb2.DockerStatsMonitorDTO.FromString,
                )
        self.stopDockerStatsMonitor = channel.unary_unary(
                '/DockerStatsManager/stopDockerStatsMonitor',
                request_serializer=docker__stats__manager__pb2.StopDockerStatsMonitorMsg.SerializeToString,
                response_deserializer=docker__stats__manager__pb2.DockerStatsMonitorDTO.FromString,
                )
        self.startDockerStatsMonitor = channel.unary_unary(
                '/DockerStatsManager/startDockerStatsMonitor',
                request_serializer=docker__stats__manager__pb2.StartDockerStatsMonitorMsg.SerializeToString,
                response_deserializer=docker__stats__manager__pb2.DockerStatsMonitorDTO.FromString,
                )


class DockerStatsManagerServicer(object):
    """Interface exported by the server
    """

    def getDockerStatsMonitorStatus(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopDockerStatsMonitor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startDockerStatsMonitor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DockerStatsManagerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'getDockerStatsMonitorStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.getDockerStatsMonitorStatus,
                    request_deserializer=docker__stats__manager__pb2.GetDockerStatsMonitorStatusMsg.FromString,
                    response_serializer=docker__stats__manager__pb2.DockerStatsMonitorDTO.SerializeToString,
            ),
            'stopDockerStatsMonitor': grpc.unary_unary_rpc_method_handler(
                    servicer.stopDockerStatsMonitor,
                    request_deserializer=docker__stats__manager__pb2.StopDockerStatsMonitorMsg.FromString,
                    response_serializer=docker__stats__manager__pb2.DockerStatsMonitorDTO.SerializeToString,
            ),
            'startDockerStatsMonitor': grpc.unary_unary_rpc_method_handler(
                    servicer.startDockerStatsMonitor,
                    request_deserializer=docker__stats__manager__pb2.StartDockerStatsMonitorMsg.FromString,
                    response_serializer=docker__stats__manager__pb2.DockerStatsMonitorDTO.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'DockerStatsManager', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class DockerStatsManager(object):
    """Interface exported by the server
    """

    @staticmethod
    def getDockerStatsMonitorStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/DockerStatsManager/getDockerStatsMonitorStatus',
            docker__stats__manager__pb2.GetDockerStatsMonitorStatusMsg.SerializeToString,
            docker__stats__manager__pb2.DockerStatsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopDockerStatsMonitor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/DockerStatsManager/stopDockerStatsMonitor',
            docker__stats__manager__pb2.StopDockerStatsMonitorMsg.SerializeToString,
            docker__stats__manager__pb2.DockerStatsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startDockerStatsMonitor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/DockerStatsManager/startDockerStatsMonitor',
            docker__stats__manager__pb2.StartDockerStatsMonitorMsg.SerializeToString,
            docker__stats__manager__pb2.DockerStatsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from typing import Dict, Any
import csle_collector.traffic_manager.traffic_manager_pb2


class TrafficManagerUtil:
    """
    Class with utility functions related to the Traffic Manager
    """

    @staticmethod
    def traffic_dto_to_dict(traffic_dto: csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO) \
            -> Dict[str, Any]:
        """
        Converts a TrafficDTO to a dict

        :param traffic_dto: the dto to convert
        :return: a dict representation of the DTO
        """
        d = {}
        d["running"] = traffic_dto.running
        d["script"] = traffic_dto.script
        return d

    @staticmethod
    def traffic_dto_from_dict(d: Dict[str, Any]) -> csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO:
        """
        Converts a dict representation of a TrafficDTO to a DTO

        :param d: the dict to convert
        :return: the converted DTO
        """
        traffic_dto = csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO()
        traffic_dto.running = d["running"]
        traffic_dto.script = d["script"]
        return traffic_dto

    @staticmethod
    def traffic_dto_empty() -> csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO:
        """
        :return: an empty TrafficDTO
        """
        traffic_dto = csle_collector.traffic_manager.traffic_manager_pb2.TrafficDTO()
        traffic_dto.running = False
        traffic_dto.script = ""
        return traffic_dto



import logging
import threading
import time
from concurrent import futures
import grpc
import socket
import subprocess
import json
import netifaces
import requests
import csle_collector.ryu_manager.ryu_manager_pb2_grpc
import csle_collector.ryu_manager.ryu_manager_pb2
import csle_collector.constants.constants as constants


class FailureDetector(threading.Thread):
    """
    Thread representing a client
    """

    def __init__(self, sleep_time: int, ip: str, ryu_web_port: int, ryu_port: int, controller: str, kafka_ip: str,
                 kafka_port: int, time_step_len: int) -> None:
        """
        Initializes the failure detector

        :param sleep_time: the period to check  for failures
        :param ip: the ip of the host
        :param ryu_web_port: the Ryu web port
        :param ryu_port: the Ryu port
        :param controller: the Ryu controller module
        :param kafka_ip: the Kafka IP
        :param kafka_port: the Kafka port
        :param time_step_len: the time-step length for Kafka logging
        """
        threading.Thread.__init__(self)
        self.sleep_time = sleep_time
        self.ip = ip
        self.ryu_web_port = ryu_web_port
        self.ryu_port = ryu_port
        self.controller = controller
        self.kafka_ip = kafka_ip
        self.kafka_port = kafka_port
        self.time_step_len = time_step_len

    def run(self) -> None:
        """
        The failure detector loop

        :return: None
        """
        done = False
        while not done:
            status_url = f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}" \
                         f"{constants.RYU.STATUS_PRODUCER_HTTP_RESOURCE}"
            try:
                requests.get(status_url, timeout=constants.RYU.REQUEST_TIMEOUT_S)
            except Exception:
                logging.info("Restarting Ryu..")
                cmd = constants.RYU.STOP_RYU_CONTROLLER
                logging.info(f"Stopping ryu with command: {cmd}")
                result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
                logging.info(f"Stdout: {result.stdout}, stderr: {result.stderr}")
                cmd = constants.RYU.STOP_RYU_CONTROLLER_MANAGER
                logging.info(f"Stopping ryu with command: {cmd}")
                result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
                logging.info(f"Stdout: {result.stdout}, stderr: {result.stderr}")
                cmd = constants.RYU.START_RYU_CONTROLLER.format(self.ryu_port, self.ryu_web_port, self.controller)
                logging.info(f"Starting RYU controller with command: {cmd}")
                subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
                time.sleep(5)
                start_url = f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}" \
                            f"{constants.RYU.START_PRODUCER_HTTP_RESOURCE}"
                logging.info(f"Starting the RYU monitor by sending a PUT request to: {start_url}")
                requests.put(start_url, data=json.dumps({constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: self.kafka_ip,
                                                         constants.RYU.TIME_STEP_LEN_SECONDS: self.time_step_len}),
                             timeout=constants.RYU.REQUEST_TIMEOUT_S)
                logging.info("RYU monitor started")
            time.sleep(self.sleep_time)


class RyuManagerServicer(csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerServicer):
    """
    gRPC server for managing a Ryu controller. Allows to start/stop the ryu controller remotely and also to query the
    state of the controller
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        logging.basicConfig(filename=f"{constants.LOG_FILES.RYU_MANAGER_LOG_DIR}"
                                     f"{constants.LOG_FILES.RYU_MANAGER_LOG_FILE}", level=logging.INFO)
        self.hostname = socket.gethostname()
        try:
            self.ip = netifaces.ifaddresses(constants.INTERFACES.ETH0)[netifaces.AF_INET][0][constants.INTERFACES.ADDR]
        except Exception:
            self.ip = socket.gethostbyname(self.hostname)
        self.ryu_port = 6633
        self.ryu_web_port = 8080
        self.controller = ""
        self.kafka_ip = ""
        self.kafka_port = 9092
        self.time_step_len = 30
        self.fd = None
        logging.info(f"Setting up RyuManager hostname: {self.hostname} ip: {self.ip}")

    def _get_ryu_status(self) -> bool:
        """
        Utility method to get the status of Ryu

        :return: status
        """
        cmd = constants.RYU.CHECK_IF_RYU_CONTROLLER_IS_RUNNING
        logging.info(f"Checking if Ryu controller is running by executing command: {cmd}")
        process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        stdout, stderr = process.communicate()
        if stdout is not None:
            stdout = stdout.decode()
        if stderr is not None:
            stderr = stderr.decode()
        logging.info(f"Stdout: {stdout}, Stderr: {stderr}")
        running = constants.RYU.SEARCH_CONTROLLER in stdout
        logging.info(f"Running: {running}")
        return running

    def _get_monitor_status(self) -> bool:
        """
        Utility method to get the status of the monitor

        :return: status
        """
        status_url = f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}" \
                     f"{constants.RYU.STATUS_PRODUCER_HTTP_RESOURCE}"
        logging.info(f"Checking monitor status by sending a request to: {status_url}")
        try:
            response = requests.get(status_url, timeout=constants.RYU.REQUEST_TIMEOUT_S)
        except Exception as e:
            logging.info(f"Timeout trying to check monitor status: {str(e)}, {repr(e)}")
            logging.info("Restarting Ryu..")
            cmd = constants.RYU.STOP_RYU_CONTROLLER
            logging.info(f"Stopping ryu with command: {cmd}")
            result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
            logging.info(f"Stdout: {result.stdout}, stderr: {result.stderr}")
            cmd = constants.RYU.STOP_RYU_CONTROLLER_MANAGER
            logging.info(f"Stopping ryu with command: {cmd}")
            result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
            logging.info(f"Stdout: {result.stdout}, stderr: {result.stderr}")
            cmd = constants.RYU.START_RYU_CONTROLLER.format(self.ryu_port, self.ryu_web_port, self.controller)
            logging.info(f"Starting RYU controller with command: {cmd}")
            subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            start_url = f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}" \
                        f"{constants.RYU.START_PRODUCER_HTTP_RESOURCE}"
            logging.info(f"Starting the RYU monitor by sending a PUT request to: {start_url}")
            requests.put(start_url, data=json.dumps({constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: self.kafka_ip,
                                                     constants.RYU.TIME_STEP_LEN_SECONDS: self.time_step_len}),
                         timeout=constants.RYU.REQUEST_TIMEOUT_S)
            response = requests.get(status_url, timeout=constants.RYU.REQUEST_TIMEOUT_S)

        logging.info(f"Response: {response.json()}")
        monitor_running = response.json()[constants.RYU.PRODUCER_RUNNING]
        return monitor_running

    def getRyuStatus(self, request: csle_collector.ryu_manager.ryu_manager_pb2.GetRyuStatusMsg,
                     context: grpc.ServicerContext) \
            -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
        """
        Gets the state of the Ryu controller

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the Ryu server
        """
        ryu_running = self._get_ryu_status()
        monitor_running = False
        if ryu_running:
            monitor_running = self._get_monitor_status()
        ryu_dto = csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO(ryu_running=ryu_running,
                                                                    monitor_running=monitor_running,
                                                                    port=self.ryu_port,
                                                                    web_port=self.ryu_web_port,
                                                                    controller=self.controller,
                                                                    kafka_ip=self.kafka_ip,
                                                                    kafka_port=self.kafka_port,
                                                                    time_step_len=self.time_step_len)
        return ryu_dto

    def stopRyu(self, request: csle_collector.ryu_manager.ryu_manager_pb2.StopRyuMsg,
                context: grpc.ServicerContext):
        """
        Stops the Ryu controller

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the ryu server
        """
        cmd = constants.RYU.STOP_RYU_CONTROLLER
        logging.info(f"Stopping ryu with command: {cmd}")
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        logging.info(f"Stdout: {result.stdout}, stderr: {result.stderr}")
        cmd = constants.RYU.STOP_RYU_CONTROLLER_MANAGER
        logging.info(f"Stopping ryu with command: {cmd}")
        result = subprocess.run(cmd.split(" "), capture_output=True, text=True)
        logging.info(f"Stdout: {result.stdout}, stderr: {result.stderr}")
        return csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO(ryu_running=False, monitor_running=False,
                                                                 port=self.ryu_port,
                                                                 web_port=self.ryu_web_port,
                                                                 controller=self.controller,
                                                                 kafka_ip=self.kafka_ip,
                                                                 kafka_port=self.kafka_port,
                                                                 time_step_len=self.time_step_len)

    def startRyu(self, request: csle_collector.ryu_manager.ryu_manager_pb2.StartRyuMsg,
                 context: grpc.ServicerContext) -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
        """
        Starts the ryu server

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the kafka server
        """
        logging.info(f"Starting Ryu, ryu port: {request.port}, ryu_web_port: {request.web_port}, "
                     f"controller: {request.controller}")
        self.ryu_port = request.port
        self.ryu_web_port = request.web_port
        self.controller = request.controller

        # Check if controller is already running
        ryu_running = self._get_ryu_status()
        if not ryu_running:
            # Stop old background job if running
            cmd = constants.RYU.STOP_RYU_CONTROLLER
            subprocess.run(cmd.split(" "), capture_output=True, text=True)
            cmd = constants.RYU.STOP_RYU_CONTROLLER_MANAGER
            subprocess.run(cmd.split(" "), capture_output=True, text=True)
            if self.fd is not None:
                self.fd.done = True
                self.fd = None
            cmd = constants.RYU.START_RYU_CONTROLLER.format(self.ryu_port, self.ryu_web_port, self.controller)
            logging.info(f"Starting RYU controller with command: {cmd}")
            subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            logging.info("Starting the failure detector thread")
            time.sleep(2)
            fd = FailureDetector(sleep_time=30, ip=self.ip, ryu_web_port=self.ryu_web_port, ryu_port=self.ryu_port,
                                 controller=self.controller, kafka_ip=self.kafka_ip, kafka_port=self.kafka_port,
                                 time_step_len=self.time_step_len)
            fd.start()

        ryu_dto = csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO(ryu_running=True, monitor_running=False,
                                                                    port=self.ryu_port,
                                                                    web_port=self.ryu_web_port,
                                                                    controller=self.controller,
                                                                    kafka_ip=self.kafka_ip,
                                                                    kafka_port=self.kafka_port,
                                                                    time_step_len=self.time_step_len)
        return ryu_dto

    def stopRyuMonitor(self, request: csle_collector.ryu_manager.ryu_manager_pb2.StopRyuMsg,
                       context: grpc.ServicerContext):
        """
        Stops the Ryu monitor

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the ryu server
        """
        ryu_running = self._get_ryu_status()
        logging.info(f"Stopping Ryu monitor, sending a request to: "
                     f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}"
                     f"{constants.RYU.STOP_PRODUCER_HTTP_RESOURCE} ryu_running: {ryu_running}")
        if ryu_running:
            requests.post(f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}"
                          f"{constants.RYU.STOP_PRODUCER_HTTP_RESOURCE}", timeout=constants.RYU.REQUEST_TIMEOUT_S)
        return csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO(ryu_running=ryu_running, monitor_running=False,
                                                                 port=self.ryu_port,
                                                                 web_port=self.ryu_web_port,
                                                                 controller=self.controller,
                                                                 kafka_ip=self.kafka_ip,
                                                                 kafka_port=self.kafka_port,
                                                                 time_step_len=self.time_step_len)

    def startRyuMonitor(self, request: csle_collector.ryu_manager.ryu_manager_pb2.StartRyuMsg,
                        context: grpc.ServicerContext) -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
        """
        Starts the ryu monitor

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the kafka server
        """
        ryu_running = self._get_ryu_status()
        logging.info("Starting Ryu monitor, sending a request to:"
                     f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}"
                     f"{constants.RYU.START_PRODUCER_HTTP_RESOURCE}, kafka_ip: {request.kafka_ip}, "
                     f"kafka port: {request.kafka_port}, time_step_len: {request.time_step_len},"
                     f"ryu_running: {ryu_running}")
        self.kafka_ip = request.kafka_ip
        self.kafka_port = request.kafka_port
        self.time_step_len = request.time_step_len
        monitor_running = False
        if ryu_running:
            response = requests.put(
                f"{constants.HTTP.HTTP_PROTOCOL_PREFIX}{self.ip}:{self.ryu_web_port}"
                f"{constants.RYU.START_PRODUCER_HTTP_RESOURCE}",
                data=json.dumps({constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: self.kafka_ip,
                                 constants.RYU.TIME_STEP_LEN_SECONDS: self.time_step_len}),
                timeout=constants.RYU.REQUEST_TIMEOUT_S)
            monitor_running = response.status_code == constants.HTTP.OK_RESPONSE_CODE
        ryu_dto = csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO(ryu_running=ryu_running,
                                                                    monitor_running=monitor_running,
                                                                    port=self.ryu_port,
                                                                    web_port=self.ryu_web_port,
                                                                    controller=self.controller,
                                                                    kafka_ip=self.kafka_ip,
                                                                    kafka_port=self.kafka_port,
                                                                    time_step_len=self.time_step_len)
        return ryu_dto


def serve(port: int = 50042, log_dir: str = "/", max_workers: int = 10,
          log_file_name: str = "ryu_manager.log") -> None:
    """
    Starts the gRPC server for managing the ryu controller

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of GRPC workers
    :return: None
    """
    constants.LOG_FILES.RYU_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.RYU_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.ryu_manager.ryu_manager_pb2_grpc.add_RyuManagerServicer_to_server(
        RyuManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"RyuManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import csle_collector.snort_ids_manager.snort_ids_manager_pb2 as snort__ids__manager__pb2


class SnortIdsManagerStub(object):
    """Interface exported by the server
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.getSnortIdsAlerts = channel.unary_unary(
                '/SnortIdsManager/getSnortIdsAlerts',
                request_serializer=snort__ids__manager__pb2.GetSnortIdsAlertsMsg.SerializeToString,
                response_deserializer=snort__ids__manager__pb2.SnortIdsLogDTO.FromString,
                )
        self.stopSnortIdsMonitor = channel.unary_unary(
                '/SnortIdsManager/stopSnortIdsMonitor',
                request_serializer=snort__ids__manager__pb2.StopSnortIdsMonitorMsg.SerializeToString,
                response_deserializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
                )
        self.startSnortIdsMonitor = channel.unary_unary(
                '/SnortIdsManager/startSnortIdsMonitor',
                request_serializer=snort__ids__manager__pb2.StartSnortIdsMonitorMsg.SerializeToString,
                response_deserializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
                )
        self.getSnortIdsMonitorStatus = channel.unary_unary(
                '/SnortIdsManager/getSnortIdsMonitorStatus',
                request_serializer=snort__ids__manager__pb2.GetSnortIdsMonitorStatusMsg.SerializeToString,
                response_deserializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
                )
        self.stopSnortIds = channel.unary_unary(
                '/SnortIdsManager/stopSnortIds',
                request_serializer=snort__ids__manager__pb2.StopSnortIdsMsg.SerializeToString,
                response_deserializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
                )
        self.startSnortIds = channel.unary_unary(
                '/SnortIdsManager/startSnortIds',
                request_serializer=snort__ids__manager__pb2.StartSnortIdsMsg.SerializeToString,
                response_deserializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
                )


class SnortIdsManagerServicer(object):
    """Interface exported by the server
    """

    def getSnortIdsAlerts(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopSnortIdsMonitor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startSnortIdsMonitor(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def getSnortIdsMonitorStatus(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def stopSnortIds(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def startSnortIds(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SnortIdsManagerServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'getSnortIdsAlerts': grpc.unary_unary_rpc_method_handler(
                    servicer.getSnortIdsAlerts,
                    request_deserializer=snort__ids__manager__pb2.GetSnortIdsAlertsMsg.FromString,
                    response_serializer=snort__ids__manager__pb2.SnortIdsLogDTO.SerializeToString,
            ),
            'stopSnortIdsMonitor': grpc.unary_unary_rpc_method_handler(
                    servicer.stopSnortIdsMonitor,
                    request_deserializer=snort__ids__manager__pb2.StopSnortIdsMonitorMsg.FromString,
                    response_serializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.SerializeToString,
            ),
            'startSnortIdsMonitor': grpc.unary_unary_rpc_method_handler(
                    servicer.startSnortIdsMonitor,
                    request_deserializer=snort__ids__manager__pb2.StartSnortIdsMonitorMsg.FromString,
                    response_serializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.SerializeToString,
            ),
            'getSnortIdsMonitorStatus': grpc.unary_unary_rpc_method_handler(
                    servicer.getSnortIdsMonitorStatus,
                    request_deserializer=snort__ids__manager__pb2.GetSnortIdsMonitorStatusMsg.FromString,
                    response_serializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.SerializeToString,
            ),
            'stopSnortIds': grpc.unary_unary_rpc_method_handler(
                    servicer.stopSnortIds,
                    request_deserializer=snort__ids__manager__pb2.StopSnortIdsMsg.FromString,
                    response_serializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.SerializeToString,
            ),
            'startSnortIds': grpc.unary_unary_rpc_method_handler(
                    servicer.startSnortIds,
                    request_deserializer=snort__ids__manager__pb2.StartSnortIdsMsg.FromString,
                    response_serializer=snort__ids__manager__pb2.SnortIdsMonitorDTO.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'SnortIdsManager', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class SnortIdsManager(object):
    """Interface exported by the server
    """

    @staticmethod
    def getSnortIdsAlerts(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SnortIdsManager/getSnortIdsAlerts',
            snort__ids__manager__pb2.GetSnortIdsAlertsMsg.SerializeToString,
            snort__ids__manager__pb2.SnortIdsLogDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopSnortIdsMonitor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SnortIdsManager/stopSnortIdsMonitor',
            snort__ids__manager__pb2.StopSnortIdsMonitorMsg.SerializeToString,
            snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startSnortIdsMonitor(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SnortIdsManager/startSnortIdsMonitor',
            snort__ids__manager__pb2.StartSnortIdsMonitorMsg.SerializeToString,
            snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def getSnortIdsMonitorStatus(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SnortIdsManager/getSnortIdsMonitorStatus',
            snort__ids__manager__pb2.GetSnortIdsMonitorStatusMsg.SerializeToString,
            snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def stopSnortIds(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SnortIdsManager/stopSnortIds',
            snort__ids__manager__pb2.StopSnortIdsMsg.SerializeToString,
            snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def startSnortIds(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/SnortIdsManager/startSnortIds',
            snort__ids__manager__pb2.StartSnortIdsMsg.SerializeToString,
            snort__ids__manager__pb2.SnortIdsMonitorDTO.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



import time
from typing import Tuple, List
import logging
import subprocess
import os
from concurrent import futures
import grpc
import socket
import netifaces
import confluent_kafka
import confluent_kafka.admin
import csle_collector.kafka_manager.kafka_manager_pb2_grpc
import csle_collector.kafka_manager.kafka_manager_pb2
import csle_collector.constants.constants as constants


class KafkaManagerServicer(csle_collector.kafka_manager.kafka_manager_pb2_grpc.KafkaManagerServicer):
    """
    gRPC server for managing a Kafka server. Allows to start/stop the kafka server remotely and also to query the
    state of the server and create/delete topics.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        logging.basicConfig(filename=f"{constants.LOG_FILES.KAFKA_MANAGER_LOG_DIR}"
                                     f"{constants.LOG_FILES.KAFKA_MANAGER_LOG_FILE}", level=logging.INFO)
        self.hostname = socket.gethostname()
        try:
            self.ip = netifaces.ifaddresses(constants.INTERFACES.ETH0)[netifaces.AF_INET][0][constants.INTERFACES.ADDR]
        except Exception:
            self.ip = socket.gethostbyname(self.hostname)
        self.conf = {constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.ip}:{constants.KAFKA.PORT}",
                     constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        logging.info(f"Setting up KafkaManager hostname: {self.hostname} ip: {self.ip}")

    def _get_kafka_status_and_topics(self) -> Tuple[bool, List[str]]:
        """
        Utility method to get the status of Kafka and existing topics

        :return: status and list of topics
        """
        status_output = subprocess.run(constants.KAFKA.KAFKA_STATUS.split(" "), capture_output=True, text=True).stdout
        running = not ("not" in status_output)
        topics = []
        if running:
            client = confluent_kafka.admin.AdminClient(self.conf)
            try:
                cluster_metadata = client.list_topics(timeout=1)
                for k, v in cluster_metadata.topics.items():
                    topics.append(k)
            except Exception as e:
                logging.info(f"There was an exception listing the Kafka topics: {str(e)}, {repr(e)}")
        return running, topics

    def getKafkaStatus(self, request: csle_collector.kafka_manager.kafka_manager_pb2.GetKafkaStatusMsg,
                       context: grpc.ServicerContext) \
            -> csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO:
        """
        Gets the state of the kafka server

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the kafka server
        """
        running, topics = self._get_kafka_status_and_topics()
        kafka_dto = csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO(running=running, topics=topics)
        return kafka_dto

    def stopKafka(self, request: csle_collector.kafka_manager.kafka_manager_pb2.StopKafkaMsg,
                  context: grpc.ServicerContext):
        """
        Stops the kafka server

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the kafka server
        """
        logging.info("Stopping kafka")
        os.system(constants.KAFKA.KAFKA_STOP)
        return csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO(running=False, topics=[])

    def startKafka(self, request: csle_collector.kafka_manager.kafka_manager_pb2.StartKafkaMsg,
                   context: grpc.ServicerContext) -> csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO:
        """
        Starts the kafka server

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the kafka server
        """
        logging.info("Starting kafka")
        os.system(constants.KAFKA.KAFKA_START)
        kafka_dto = csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO(running=True, topics=[])
        return kafka_dto

    def hours_to_ms(self, hours: int) -> float:
        """
        Convert hours to ms

        :param hours: the hours to convert
        :return: the ms
        """
        return int((((hours * 1000) * 60) * 60))

    def createTopic(self, request: csle_collector.kafka_manager.kafka_manager_pb2.CreateTopicMsg,
                    context: grpc.ServicerContext) -> csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO:
        """
        Creates a new Kafka topic

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the kafka server
        """
        logging.info(f"Creating topic: {request.name}, partitions:{request.partitions}, replicas:{request.replicas}, "
                     f"retention hours: {request.retention_time_hours}")
        running, topics = self._get_kafka_status_and_topics()
        client = confluent_kafka.admin.AdminClient(self.conf)
        config = {
            constants.KAFKA.RETENTION_MS_CONFIG_PROPERTY: self.hours_to_ms(request.retention_time_hours)}
        new_topic = confluent_kafka.admin.NewTopic(
            request.name, request.partitions, request.replicas,
            config=config)
        client.create_topics([new_topic])
        time.sleep(5)
        kafka_dto = csle_collector.kafka_manager.kafka_manager_pb2.KafkaDTO(running=True,
                                                                            topics=topics + [request.name])
        return kafka_dto


def serve(port: int = 50051, log_dir: str = "/", max_workers: int = 10,
          log_file_name: str = "kafka_manager.log") -> None:
    """
    Starts the gRPC server for managing the kafka server

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of GRPC workers
    :return: None
    """
    constants.LOG_FILES.KAFKA_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.KAFKA_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.kafka_manager.kafka_manager_pb2_grpc.add_KafkaManagerServicer_to_server(
        KafkaManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"KafkaManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



from typing import Tuple, List, Dict
import time
import os
import logging
import threading
import docker
from concurrent import futures
import grpc
import socket
from confluent_kafka import Producer
from collections import deque
import csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc
import csle_collector.docker_stats_manager.docker_stats_manager_pb2
from csle_collector.docker_stats_manager.docker_stats_manager_pb2 import ContainerIp
from csle_collector.docker_stats_manager.docker_stats import DockerStats
from csle_collector.docker_stats_manager.docker_stats_util import DockerStatsUtil
import csle_collector.constants.constants as constants


class DockerStatsThread(threading.Thread):
    """
    Thread that collects performance statistics of Docker containers
    """

    def __init__(self, container_names_and_ips: List[ContainerIp], emulation: str, execution_first_ip_octet: int,
                 kafka_ip: str, stats_queue_maxsize: int, time_step_len_seconds: int, kafka_port: int) -> None:
        """
        Initializes the thread

        :param container_names_and_ips: list of container names and ips to monitor
        :param emulation: name of the emulation to monitor
        :param kafka_ip: the ip of the Kafka server to produce stats to
        :param kafka_port: the port of the Kafka server to produce stats to
        :param stats_queue_maxsize: max length of the queue before sending to Kafka
        :param time_step_len_seconds: the length of a time-step before sending stats to Kafka
        """
        threading.Thread.__init__(self)
        self.container_names_and_ips = container_names_and_ips
        self.emulation = emulation
        self.execution_first_ip_octet = execution_first_ip_octet
        self.client_1 = docker.from_env()
        self.client2 = docker.APIClient(base_url=constants.DOCKER_STATS.UNIX_DOCKER_SOCK_URL)
        self.containers = self.client_1.containers.list()
        self.container_names = list(map(lambda x: x.container, self.container_names_and_ips))
        self.containers = list(filter(lambda x: x.name in self.container_names, self.containers))
        streams = []
        for container in self.containers:
            stream = container.stats(decode=True, stream=True)
            streams.append((stream, container))
        self.stats_queue_maxsize = stats_queue_maxsize
        self.streams = streams
        self.stats_queues = {}
        self.time_step_len_seconds = time_step_len_seconds
        self.kafka_ip = kafka_ip
        self.kafka_port = kafka_port
        self.hostname = socket.gethostname()
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 80))
        self.ip = s.getsockname()[0]
        self.conf = {constants.KAFKA.BOOTSTRAP_SERVERS_PROPERTY: f"{self.kafka_ip}:{self.kafka_port}",
                     constants.KAFKA.CLIENT_ID_PROPERTY: self.hostname}
        self.producer = Producer(**self.conf)
        self.stopped = False
        logging.info(f"Producer thread starting, emulation:{self.emulation}, "
                     f"execution_first_ip_octet: {execution_first_ip_octet}, kafka ip: {self.kafka_ip}, "
                     f"kafka port:{self.kafka_port}, time_step_len_seconds: {self.time_step_len_seconds}, "
                     f"container and ips:{self.container_names_and_ips}")

    def run(self) -> None:
        """
        Main loop of the thread

        :return: None
        """
        start = time.time()
        while not self.stopped:
            time.sleep(5)
            try:
                if time.time() - start >= self.time_step_len_seconds:
                    aggregated_stats, avg_stats_dict = self.compute_averages()
                    record = aggregated_stats.to_kafka_record(ip=self.ip)
                    self.producer.produce(constants.KAFKA_CONFIG.DOCKER_STATS_TOPIC_NAME, record)
                    self.producer.poll(0)
                    for k, v in avg_stats_dict.items():
                        ip = self.get_ip(k)
                        record = v.to_kafka_record(ip=ip)
                        self.producer.produce(constants.KAFKA_CONFIG.DOCKER_HOST_STATS_TOPIC_NAME, record)
                        self.producer.poll(0)
                    self.stats_queues = {}
                    start = time.time()

                for stream, container in self.streams:
                    stats_dict = next(stream)
                    parsed_stats = DockerStatsUtil.parse_stats(stats_dict, container.name)
                    if parsed_stats.container_name not in self.stats_queues:
                        self.stats_queues[parsed_stats.container_name] = deque([], maxlen=self.stats_queue_maxsize)
                    self.stats_queues[parsed_stats.container_name].append(parsed_stats)
            except BaseException as e:
                logging.warning(f"Exception in monitor thread for emulation: "
                                f"{self.emulation}, exception: {str(e)}, {repr(e)}")

    def get_ip(self, container_name: str) -> str:
        """
        Gets the ip of a given container name

        :param container_name: the name of the container
        :return: the ip of the container
        """
        for name_ip in self.container_names_and_ips:
            if name_ip.container == container_name:
                return name_ip.ip
        return "-"

    def compute_averages(self) -> Tuple[DockerStats, Dict[str, DockerStats]]:
        """
        Compute averages and aggregates of the list of metrics
        :return: the average and aggregate metrics
        """
        avg_stats = {}
        avg_stats_l = []
        for k, v in self.stats_queues.items():
            avg_stat = DockerStats.compute_averages(list(v))
            avg_stats[k] = avg_stat
            avg_stats_l.append(avg_stat)
        aggregated_stats = DockerStats.compute_averages(avg_stats_l)
        aggregated_stats.pids = float("{:.1f}".format(aggregated_stats.pids * len(avg_stats_l)))
        aggregated_stats.mem_current = float("{:.1f}".format(aggregated_stats.mem_current * len(avg_stats_l)))
        aggregated_stats.mem_total = float("{:.1f}".format(aggregated_stats.mem_total * len(avg_stats_l)))
        aggregated_stats.blk_read = float("{:.1f}".format(aggregated_stats.blk_read * len(avg_stats_l)))
        aggregated_stats.blk_write = float("{:.1f}".format(aggregated_stats.blk_write * len(avg_stats_l)))
        aggregated_stats.net_rx = float("{:.1f}".format(aggregated_stats.net_rx * len(avg_stats_l)))
        aggregated_stats.net_tx = float("{:.1f}".format(aggregated_stats.net_tx * len(avg_stats_l)))
        return aggregated_stats, avg_stats


class DockerStatsManagerServicer(csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc.
                                 DockerStatsManagerServicer):
    """
    gRPC server for managing a the docker stats monitor server.
    Allows to start/stop the docker stats monitor remotely and also to query the
    state of the server.
    """

    def __init__(self) -> None:
        """
        Initializes the server
        """
        file_name = constants.LOG_FILES.DOCKER_STATS_MANAGER_LOG_FILE
        dir = constants.LOG_FILES.DOCKER_STATS_MANAGER_LOG_DIR
        logfile = os.path.join(dir, file_name)
        logging.basicConfig(filename=logfile, level=logging.INFO)
        self.docker_stats_monitor_threads = []
        self.hostname = socket.gethostname()
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 80))
        self.ip = s.getsockname()[0]
        logging.info(f"Setting up DockerStatsManager, hostname: {self.hostname}, ip: {self.ip}")

    def getDockerStatsMonitorStatus(
            self, request: csle_collector.docker_stats_manager.docker_stats_manager_pb2.GetDockerStatsMonitorStatusMsg,
            context: grpc.ServicerContext) \
            -> csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO:
        """
        Gets the state of the docker stats monitors

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the docker stats manager
        """
        new_docker_stats_monitor_threads = []
        emulations = []
        emulation_executions = []
        for dsmt in self.docker_stats_monitor_threads:
            if dsmt.is_alive() and not dsmt.stopped:
                new_docker_stats_monitor_threads.append(dsmt)
                emulations.append(dsmt.emulation)
                emulation_executions.append(dsmt.execution_first_ip_octet)
            else:
                dsmt.stopped = True
        emulations = list(set(emulations))
        emulation_executions = list(set(emulation_executions))
        self.docker_stats_monitor_threads = new_docker_stats_monitor_threads
        docker_stats_monitor_dto = csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO(
            num_monitors=len(self.docker_stats_monitor_threads), emulations=emulations,
            emulation_executions=emulation_executions)
        return docker_stats_monitor_dto

    def stopDockerStatsMonitor(
            self, request: csle_collector.docker_stats_manager.docker_stats_manager_pb2.StopDockerStatsMonitorMsg,
            context: grpc.ServicerContext):
        """
        Stops the docker stats monitor server

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the docker stats monitor server
        """
        logging.info(f"Stopping the docker stats monitor for emulation:{request.emulation}")

        new_docker_stats_monitor_threads = []
        for dsmt in self.docker_stats_monitor_threads:
            if dsmt.emulation == request.emulation \
                    and dsmt.execution_first_ip_octet == request.execution_first_ip_octet:
                dsmt.stopped = True
            else:
                if dsmt.is_alive() and not dsmt.stopped:
                    new_docker_stats_monitor_threads.append(dsmt)
        self.docker_stats_monitor_threads = new_docker_stats_monitor_threads
        return csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO(
            num_monitors=len(self.docker_stats_monitor_threads))

    def startDockerStatsMonitor(
            self, request: csle_collector.docker_stats_manager.docker_stats_manager_pb2.StartDockerStatsMonitorMsg,
            context: grpc.ServicerContext) \
            -> csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO:
        """
        Starts a new docker stats monitor

        :param request: the gRPC request
        :param context: the gRPC context
        :return: a clients DTO with the state of the docker stats monitor
        """
        logging.info(f"Starting the docker stats monitor for emulation:{request.emulation}")

        # Stop any existing thread with the same name
        new_docker_stats_monitor_threads = []
        for dsmt in self.docker_stats_monitor_threads:
            if dsmt.emulation == request.emulation and \
                    dsmt.execution_first_ip_octet == request.execution_first_ip_octet:
                dsmt.stopped = True
            else:
                if dsmt.is_alive():
                    new_docker_stats_monitor_threads.append(dsmt)
        self.docker_stats_monitor_threads = new_docker_stats_monitor_threads

        docker_stats_monitor_thread = DockerStatsThread(request.containers, request.emulation,
                                                        request.execution_first_ip_octet, request.kafka_ip,
                                                        request.stats_queue_maxsize, request.time_step_len_seconds,
                                                        request.kafka_port)
        docker_stats_monitor_thread.start()
        self.docker_stats_monitor_threads.append(docker_stats_monitor_thread)
        new_docker_stats_monitor_threads = []
        for dsmt in self.docker_stats_monitor_threads:
            if dsmt.is_alive() and not dsmt.stopped:
                new_docker_stats_monitor_threads.append(dsmt)
        self.docker_stats_monitor_threads = new_docker_stats_monitor_threads
        return csle_collector.docker_stats_manager.docker_stats_manager_pb2.DockerStatsMonitorDTO(
            num_monitors=len(self.docker_stats_monitor_threads))


def serve(port: int = 50046, log_dir: str = "/var/log/csle/", max_workers: int = 10,
          log_file_name: str = "docker_stats_manager.log") -> None:
    """
    Starts the gRPC server for managing docker stats collection

    :param port: the port that the server will listen to
    :param log_dir: the directory to write the log file
    :param log_file_name: the file name of the log
    :param max_workers: the maximum number of parallel gRPC workers
    :return: None
    """
    constants.LOG_FILES.DOCKER_STATS_MANAGER_LOG_DIR = log_dir
    constants.LOG_FILES.DOCKER_STATS_MANAGER_LOG_FILE = log_file_name
    server = grpc.server(futures.ThreadPoolExecutor(max_workers=max_workers))
    csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc.add_DockerStatsManagerServicer_to_server(
        DockerStatsManagerServicer(), server)
    server.add_insecure_port(f'[::]:{port}')
    server.start()
    logging.info(f"DockerStatsManager Server Started, Listening on port: {port}")
    server.wait_for_termination()


# Program entrypoint
if __name__ == '__main__':
    serve()



from typing import List
import threading
import time
import subprocess


class ClientThread(threading.Thread):
    """
    Thread representing a client
    """

    def __init__(self, commands: List[str], time_step_len_seconds: float) -> None:
        """
        Initializes the client thread

        :param commands: the sequence of commands that the client will execute
        :param time_step_len_seconds: the length of a time-step in seconds
        """
        threading.Thread.__init__(self)
        self.commands = commands
        self.time_step_len_seconds = time_step_len_seconds

    def run(self) -> None:
        """
        The main function of the client. It executes a sequence of commands and then terminates

        :return: None
        """
        for cmd in self.commands:
            p = subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)
            p.communicate()
            p.wait()
            time.sleep(self.time_step_len_seconds)